/* Notes
	A plug in must not call a function, even a built in function. This will then make
	it relocatable and can be used in any ByPic project at any position. If a function
	is called this not only increases the complexity but will make it very specific to
	a device and release.
	A plug in requires a data location to be passed and will always return an integer.
	
	When the plugin is built, the .elf file along with pic32-objdump is used to produce a 
	disassembly of the code, this is placed in 'dis.tst'. The opcodes are taken from that 
	file are put in a constant {} data statement ready for the ByPic to use.
	
	This could be done manually but there is a A Python program called distobasX.py (X is a number) 
	that does this automatically. It will create 'dis.txt', parse it and output a file called 'plugin.bas'
	
	Compiler options: -mlong-calls -fPIC -mno-gpopt -mshared -mno-abicalls
*/
// 1a (13 Jan 2017)
// Added timeout
//
#include <plib.h>
#define nop()  __asm__ __volatile__ ("%(ssnop%)" : :)
// Integer pointer arithmatic, 1 is 4 bytes
#define CLR  1
#define SET  2
#define INV  3
//
// BB soft UART
// int var = *(ip+n) // read address
// int *var = (ip+n) // read /write
// memory offset    contents
//		(0)      RX PORT address - pre set to in
//		(1)      RX pin number
//		(2)      TX LAT address - pre set to out
//      (3)      TX pin number
//		(4)		timer for example tmr2
//	    (5)		1/2 bit delay value
//		(6)		tx byte to send
int bb_getc(char *c)
{
// register addresses
int *ip;
unsigned int *rxport,rxpin,*tmr,hdel,*bcount,bmax; 
unsigned int deltime;
unsigned char j=7, inbyte;

	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	rxport = (int*)*(ip);
	rxpin = 1 << *(ip+1); // mask
	tmr = (int*)*(ip+4);
	hdel = *(ip+5);

	// 1+1/2 bit delay
	*tmr = 0;
	inbyte = 0;
	deltime = (hdel * 3);
	while(*tmr < deltime);
    while(j--){
		*tmr = 0; // reset here, starts counting up now
		if(*rxport & rxpin) inbyte+=0x80;	
		inbyte >>= 1;
		deltime = hdel * 2;
		while(*tmr < deltime);
	}
	*tmr = 0; // ready for stop bit
	deltime = hdel * 2; // stop bit
	while(*tmr < deltime); // stop bit
	while(!(*rxport & rxpin)); // wait for actual stop
    return inbyte;
}

// memory offset    contents
//		(0)      RX PORT address - pre set to in
//		(1)      RX pin number
//		(2)      TX LAT address - pre set to out
//      (3)      TX pin number
//		(4)		timer for example tmr2
//	    (5)		1/2 bit delay value
//		(6)		tx byte to send
int bb_putc(char *c) {
int *ip;
unsigned int *txlat,txpin,*tmr,hdel;
unsigned int deltime;
unsigned char ch, j=0;

	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	txlat = (int*)*(ip+2);
	txpin = 1 << *(ip+3); // CLR,SET mask
	tmr = (int*)*(ip+4);
	hdel = *(ip+5);
	ch = (char)*(ip+6);


	// start bit
	*(txlat+CLR) = txpin; // low for start bit
	*tmr = 0;
	deltime = hdel * 2; // start period
	while(*tmr < deltime);
	while(j < 8) {
		if((ch >> j) & 1) *(txlat+SET) = txpin;
		else *(txlat+CLR) = txpin;
		*tmr = 0;
		deltime = hdel * 2;
		while(*tmr < deltime);
		j++;
	}
	// stop bit
	*(txlat+SET) = txpin;
	*tmr = 0;
	deltime = hdel * 2;
	while(*tmr < deltime);
	return ch;
}

void main() {}

